﻿using System.IO;
using HeuristicLab.Data;

namespace HeuristicLab.Networks.IntegratedOptimization {
  public static class TtpImporter {
    public static void ImportTtpInstance(string filePath, out DoubleMatrix tspCoordinates,
                                                   out IntValue kspCapacity,
                                                   out IntArray kspItemValues,
                                                   out IntArray kspItemWeights,
                                                   out IntArray ttpAvailability,
                                                   out DoubleValue ttpMinSpeed,
                                                   out DoubleValue ttpMaxSpeed,
                                                   out DoubleValue ttpRentingRatio) {
      int nrOfCities = 0, nrOfItems = 0, capacity = 0;
      double minSpeed = 0.0, maxSpeed = 0.0, rentingRatio = 0.0;

      using (var fs = new FileStream(filePath, FileMode.Open))
      using (var sr = new StreamReader(fs)) {
        string input;
        while ((input = sr.ReadLine()) != null && !input.Contains("NODE_COORD_SECTION")) {
          if (input.Contains("DIMENSION:")) int.TryParse(input.Replace("DIMENSION:", string.Empty), out nrOfCities);
          if (input.Contains("NUMBER OF ITEMS:")) int.TryParse(input.Replace("NUMBER OF ITEMS:", string.Empty), out nrOfItems);
          if (input.Contains("CAPACITY OF KNAPSACK:")) int.TryParse(input.Replace("CAPACITY OF KNAPSACK:", string.Empty), out capacity);
          if (input.Contains("MIN SPEED:")) double.TryParse(input.Replace("MIN SPEED:", string.Empty), out minSpeed);
          if (input.Contains("MAX SPEED:")) double.TryParse(input.Replace("MAX SPEED:", string.Empty), out maxSpeed);
          if (input.Contains("RENTING RATIO:")) double.TryParse(input.Replace("RENTING RATIO:", string.Empty), out rentingRatio);
        }

        tspCoordinates = new DoubleMatrix(nrOfCities, 2);
        kspItemValues = new IntArray(nrOfItems);
        kspItemWeights = new IntArray(nrOfItems);
        ttpAvailability = new IntArray(nrOfItems);

        // read cities
        while ((input = sr.ReadLine()) != null && !input.Contains("ITEMS SECTION")) {
          string[] data = input.Split('\t');
          int index; double x, y;
          int.TryParse(data[0], out index);
          double.TryParse(data[1], out x);
          double.TryParse(data[2], out y);
          tspCoordinates[index - 1, 0] = x;
          tspCoordinates[index - 1, 1] = y;
        }
        // read items
        while ((input = sr.ReadLine()) != null) {
          string[] data = input.Split('\t');
          int index, value, weight, city;
          int.TryParse(data[0], out index);
          int.TryParse(data[1], out value);
          int.TryParse(data[2], out weight);
          int.TryParse(data[3], out city);
          kspItemValues[index - 1] = value;
          kspItemWeights[index - 1] = weight;
          ttpAvailability[index - 1] = city - 1;
        }
      }

      kspCapacity = new IntValue(capacity);
      ttpMinSpeed = new DoubleValue(minSpeed);
      ttpMaxSpeed = new DoubleValue(maxSpeed);
      ttpRentingRatio = new DoubleValue(rentingRatio);
    }
  }
}
